;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/         
;/ Filename: JTC_NTSC_240_192_v001.s
;/
;/ Author: JT Cook - http://www.avalondreams.com
;/ 
;/ Description: 240x192 tile/sprite driver (30x24 tile map)
;/ Supports 8 16x16 sprites
;/ Does not use palette, only reads colors from tiles and sprites
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ INCLUDES 
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; Include our register and chip definitions file
.include "p24HJ256GP206.inc"

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ DEFINES
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; width and height of screen 
.equ NTSC_SCREEN_WIDTH,     160
.equ NTSC_SCREEN_HEIGHT,    192

; controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
.equ NTSC_VERTICAL_OFFSET,  40

; number of bits each pixel is encoded with
.equ NTSC_PIXELS_PER_BYTE,  4            

; Number of clocks per pixel (12 in the case of NTSC) Modifying this won't really change anything,
; you will need to change code delays to get the correct clocks per pixel change
.equ CLOCKS_PER_PIXEL, 8

; Number of lines we should repeat to reduce the vertical resolution
.equ LINE_REPT_COUNT,   1

; with the current D/A converter range 0 .. 15, 1 = 62mV
.equ NTSC_SYNC,           (0x0F)     ; 0V  
.equ NTSC_BLACK,          (0x4F)     ; 0.3V (about 30-40 IRE)

.equ NTSC_CBURST0,        (0x30) 
.equ NTSC_CBURST1,        (0x31) 
.equ NTSC_CBURST2,        (0x32) 

.equ NTSC_CHROMA_OFF, 	0x000F
.equ NTSC_OVERSCAN_CLR,	0x005F

.equ COLOR_LUMA,    (NTSC_CBURST1 + 0x40)

.equ NTSC_GREEN,    (COLOR_LUMA + 0)        
.equ NTSC_YELLOW,   (COLOR_LUMA + 2)        
.equ NTSC_ORANGE,   (COLOR_LUMA + 4)        
.equ NTSC_RED,      (COLOR_LUMA + 5)        
.equ NTSC_VIOLET,   (COLOR_LUMA + 9)        
.equ NTSC_PURPLE,   (COLOR_LUMA + 11)        
.equ NTSC_BLUE,     (COLOR_LUMA + 14)        

.equ NTSC_GRAY0,    (NTSC_BLACK + 0x10)        
.equ NTSC_GRAY1,    (NTSC_BLACK + 0x20)        
.equ NTSC_GRAY2,    (NTSC_BLACK + 0x30)        
.equ NTSC_GRAY3,    (NTSC_BLACK + 0x40)        
.equ NTSC_GRAY4,    (NTSC_BLACK + 0x50)        
.equ NTSC_GRAY5,    (NTSC_BLACK + 0x60)        
.equ NTSC_GRAY6,    (NTSC_BLACK + 0x70)        
.equ NTSC_GRAY7,    (NTSC_BLACK + 0x80)        
.equ NTSC_GRAY8,    (NTSC_BLACK + 0x90)        
.equ NTSC_GRAY9,    (NTSC_BLACK + 0xA0)        
.equ NTSC_WHITE,    (NTSC_BLACK + 0xB0) 

.equ Sprite_Ptr_Size,    0x08    ; size of sprite pointer


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ EXTERNALS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.extern _g_TileMapBasePtr       ; 16-bit pointer to tile map, allows indirection
.extern _g_SpriteList           ; 16-bit pointer to sprite table records
.extern _g_TileSet              ; 16-bit pointer to 8x8 tile bitmaps, allows indirection

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ SRAM SECTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following section describes initialized (data) near section. The reason we want
; to place these variables into the near SRAM area is so that ASM codes, that don't use many
; bits to encode data, will be able to access the variables.
.section *,data,near
_g_CurrentLine:		.int 245		        ; Line count 0->261
_g_RasterLine:      .int 0                  ; Virtual Raster count 0->96
ReptLine:	        .int LINE_REPT_COUNT	; Repeated line counter 0->2
CurrVRAMPtr:	    .int 0			        ; Line count 0->525
CurrAttribPtr:	    .int 0			        ; Points to g_AttributeBuf that is defined in the C program as a global
_g_VsyncFlag:       .int 0			        ; Flag to indicate we are in v-sync
ReptAttrib:	        .int 8			        ; Repeat attribute table increment 8 pixel lines

; Un-initialized Data sections
.section *,bss,near
;AttribScanLine: .space ((NTSC_SCREEN_WIDTH/8)*2)	; Contains pointers to the palette table for current scanline.
ScanLine: .space (256+16)	; holds scanline where we render sprites to


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ GLOBAL EXPORTS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.global __T1Interrupt   ; Declare Timer 1 ISR name global
.global _g_VsyncFlag    ; read only, Flag to indicate we are in v-sync
.global _g_CurrentLine  ; read only, current physical line 0...261
.global _g_RasterLine   ; Virtual Raster count 0->96


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ ASM FUNCTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following are code sections
.text           

__T1Interrupt:

		; Our instruction clock is running at 12*NTSC dot clock
		; so every 12 clock cycles is a normal 227 pixels
		disi #0x3FFF					; (1) Disable interupts for up to 10000 cycles

		; Save off variables we will use
		push.s						; (1) Push status registers
		push SR
		push W4						; (1) Temp variable
		push W5
		push RCOUNT					; (1) Make sure we are pushing Rcount since we are using repeat

		; Increment which line we are on
		inc _g_CurrentLine				; (1)
		mov _g_CurrentLine, W0				; (1) Set our line variable into W0

		; Test if we are done with this frame (262 lines)
		mov #261, W2					; (1)
		cp W0, W2					; (1) See if we are at line 262 = 0
		bra nz, DONE_LINE_TEST				; (1/2) if(_g_CurrentLine != 262) jump
		setm _g_CurrentLine				; (1) Reset Line counter set to 0xFFFF
DONE_LINE_TEST: 

		; Total = 10

        ; The following section of code determines what kind of line we should be 
        ; drawing to the screen. Is it a Vsync Line, Black Top/Bottom, or an
        ; Active Video line. All of this is determined by the _g_CurrentLine counter
        ; variable

        ; if(_g_CurrentLine < VSYNC_LINE_COUNT)
		mov #5, W2					; (1)
		cp W0, W2					; (1) Compare to 5
		bra gt, NOT_VSYNC_LINE				; (1/2) if(Line > 5) jump

		; Vsync line (18)
		repeat	#11					; (1)
		nop						; (1+n)
		goto DRAW_VSYNC_LINE				; (2) draw a black line to the screen

NOT_VSYNC_LINE:

		; Test to see if this is a top blanking line
		mov #NTSC_VERTICAL_OFFSET, W2			; (1)
		cp W0, W2					; (1) Compare to NTSC_VERTICAL_OFFSET
		bra ge, NOT_TOP_OVERSCAN_LINE			; (1/2) if(Line >= NTSC_VERTICAL_OFFSET) jump

		; Blank Line Top (18)
		nop						; (1)
        nop							; (1)
        clr _g_RasterLine                          		; (1) Clear out our raster line count
		nop ;mov _g_CurrentVRAMBuff, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
		nop ;mov W1, CurrVRAMPtr				; (1) copy
		nop ;mov _g_AttribTable, W1				; (1) Copy over ATTRIBTABLE = g_AttributeTable
		nop ;mov W1, CurrAttribPtr				; (1) copy
		mov #LINE_REPT_COUNT, W1			; (1) Reset repeat line back to count
		mov W1, ReptLine							; (1)
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_TOP_OVERSCAN_LINE:

		; Test to see if this is a bottom blanking line
		mov #((NTSC_SCREEN_HEIGHT*LINE_REPT_COUNT)+NTSC_VERTICAL_OFFSET), W2	; (1)
		cp W0, W2					; (1) Compare to 245
		bra lt, NOT_BOTTOM_OVERSCAN_LINE		; (1/2) if(Line < 245) jump

		; Blank Line Bottom (18)
		repeat	#2					; (1)
		nop						; (1+n)
		setm _g_VsyncFlag				; (1) set the vsync flag 
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_BOTTOM_OVERSCAN_LINE:

		; This must be an active line (18)
		clr _g_VsyncFlag				; (1) g_VsyncFlag = 0
		nop						; (1) delay
		nop						; (1) delay
		nop						; (1) delay
		goto DRAW_ACTIVE_LINE				; (2) draw an active line to the screen



; ----------------- VSYNC LINE -------------------------
DRAW_VSYNC_LINE:

		; This draws a vsync line with h-sync pulse inverted for serration
		; H-sync serration (4.7us) 236
		mov #NTSC_BLACK,W0 	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #232					; (1)
		nop						; (1+n)
		mov #NTSC_SYNC, W0	                        ; (1)
		mov W0, LATB					; (1)

		; Done with this completely
		goto FINISHED_NTSC		; Dont need to count because interrupts will keep us locked



; ----------------- OVERSCAN LINE -------------------------
DRAW_OVERSCAN_LINE:

		; This draws a overscan line with h-sync (used for blanking lines)
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #198					; (1)
		nop						; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)


		; Color burst (10 NTSC clocks) = 120 clocks
		mov #NTSC_CBURST1, W0				; (1) turn hsync color burst on at phase 0
		mov W0, LATB					; (1)
		;repeat #116					; (1)
		repeat #(108-4)				; (1)
		nop						; (1+n)


		; Post-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)


		; Overscan color
		mov #NTSC_OVERSCAN_CLR, W0			; (1) turn hsync off black with a color signal
		mov W0, LATB					; (1)
;clear sprite scanline, we do this because if the sprite scrolls off the bottom of the screen, then it will 
;draw on the top scanline otherwise
	    mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        add W3, #16, W3   ;(1) advance to the visable part of the scanline
        mov #0, W1        ;(1) value to clear scanline with
        repeat #120     ;(1)
        mov W1, [W3++]  ;(1+n) clear scanline 
;/Draw first sprite, needed here if the sprite is on the first scaneline
      ;set up scanline for all sprites (done once per scanline) - 6 cycles
        mov _g_SpriteList, W9; (1) pointer for sprite data        
        mov _g_RasterLine, W8; (1) find out which scanline we are using
        add W8, #16, W8 ; (1) add this so we can move sprite offscreen on the Y axis
        mov #0, W4     ; (1) make sure that upper 8 bits are clear for sprite Y value
        mov #0, W5     ; (1) make sure that upper 8 bits are clear for sprite X value
        mov #0, W10    ; (1) used for compare for sprite
      ;draw sprite (29) ---- First sprite (27 and 2 for goto)
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c00 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c00  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite -  16 cycles
;;      This will place sprite without regards to transparency
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1], [W3]   ;(1)

		END_SPRITE_00:
        goto Finish_Scanline_0
;-- Time to kill if we aren't drawing the first sprite
No_sprite_1c00:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(27-5-5)    ;(1)
        nop               ;(1+n)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
;        goto Setup_scanline ;(2)
        goto END_SPRITE_00 ;(2)
No_sprite_2c00:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(27-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_00 ;(2)
        Finish_Scanline_0:


		; Done with this completely
		goto FINISHED_NTSC				; Dont need to count because interrupts will keep us locked




; ----------------- ACTIVE LINE -------------------------
DRAW_ACTIVE_LINE:

;---------------------------------------------------------------
;Hsync On - 202 Clocks - Sprites 2, 3, and 4
;    Clocks: Hsync - 5 clocks, sprite setup - 7 clocks, 3 sprites - 189 clocks, goto - 1 clocks (we get an extra from not branching)
;---------------------------------------------------------------
		; ------- HSYNC ON ---------
		; 43 * X.XuS = clock cycles
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0 	            ; (1) turn hsync on
		mov W0, LATB					; (1)
        nop                             ; (1)
        nop                             ; (1)
        nop                             ; (1)
; W1 - sprite graphic data address
; W2 - sprite color value
; W3 - address for sprite scanline
; W4 - Y value for sprite
; W5 - X value for sprite
; W6 - Sprite address for that sprite scanline
; W7 - Number of sprites to draw in loop
; W8 - current scanline + 8 (to scroll off screen)
; W9 - pointer to sprite data (x pos, y pos, sprite image data)
; W10 - Zero, used for compare on drawing sprite
      ;set up scanline for all sprites (done once per scanline) - 7 cycles
        mov _g_SpriteList, W9; (1) pointer for sprite data        
        mov _g_RasterLine, W8; (1) find out which scanline we are using
        add W8, #16, W8 ; (1) add this so we can move sprite offscreen on the Y axis
        mov #0, W4     ; (1) make sure that upper 8 bits are clear for sprite Y value
        mov #0, W5     ; (1) make sure that upper 8 bits are clear for sprite X value
        mov #0, W10    ; (1) used for compare for sprite
        mov #3, W7     ; (1) how many sprites to draw in a loop
        ; 3 sprites, 189 clocks
		END_SPRITE_1:
        ;draw sprite (60) --- Second sprite (63 including branch and loop)
        add #Sprite_Ptr_Size, W9        ;(1) advance sprite pointer to get info for this sprite
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c2 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c2  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite - 48 cycles
        ;pixel 0
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+15]  ;(1)
		END_SPRITE_2:
        sub W7, #1, W7;  (1) - subtract 1 from sprite count
		bra nz, END_SPRITE_1	; (1/2) check to see if we are drawing more sprites

		;repeat #(198-189-7-2)   	; (1)
		;nop						; (1+n)
        goto PRE_BURST      ;(2)
;-- Time to kill if we aren't drawing the second or 3rd sprite
No_sprite_1c2:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(60-5-5)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
        goto END_SPRITE_2 ;(2)
No_sprite_2c2:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(60-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_2 ;(2)
;---------------------------------------------------------------
;Pre-burst - 26 clocks
;---------------------------------------------------------------
        PRE_BURST:
		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)
;---------------------------------------------------------------
;Color Burst - 108 clocks - Sprite 5
;---------------------------------------------------------------
        ; 2.5uS - 108 clock cycles
		; Color burst (10 NTSC clocks) = 80 clocks (originally 120 clocks)
		mov #NTSC_CBURST1, W0 		        ; (1) turn hsync color burst on at phase 0
		mov W0, LATB				; (1)
        ;draw sprite (60) --- Second sprite 
        add #Sprite_Ptr_Size, W9        ;(1) advance sprite pointer to get info for this sprite
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c4 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c4  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite - 48 cycles
        ;pixel 0
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+15]  ;(1)
		END_SPRITE_4:
		repeat #(108-60-2-4)				; (1)
		nop					; (1+n)
        goto POST_BURST     ;(2) 
;-- Time to kill if we aren't drawing the second or 4th sprite
No_sprite_1c4:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(60-5-5)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
        goto END_SPRITE_4 ;(2)
No_sprite_2c4:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(60-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_4 ;(2)

;---------------------------------------------------------------
;Post Burst - 69 Clocks - Sprite 6
;---------------------------------------------------------------
        ; Post-burst 1.6uS - 69
        POST_BURST:
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB			; (1)
        ;draw sprite (60) --- Second sprite 
        add #Sprite_Ptr_Size, W9        ;(1) advance sprite pointer to get info for this sprite
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c5 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c5  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite - 48 cycles
        ;pixel 0
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+15]  ;(1)
		END_SPRITE_5:
		repeat #(69-4-60-2)			; (1)
		nop						; (1+n)
        goto OVER_SCAN          ; (2)
;-- Time to kill if we aren't drawing the second or 5th sprite
No_sprite_1c5:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(60-5-5)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
        goto END_SPRITE_5 ;(2)
No_sprite_2c5:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(60-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_5 ;(2)

;---------------------------------------------------------------
;Overscan - 16 Clocks 
;---------------------------------------------------------------
		OVER_SCAN:
		;Overscan 
		; FROM HERE ON it is imparative that we stay in sync with the color clock signal
		; (i.e. we don't change any pixels unless it is an integer multiple of 8)
		; If we don't do this then color changes will not be correct.
		; Post burst
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
;		repeat #(16-4)					; (1)
;		nop						; (1+n)

;---------------------------------------------------------------
;Left Overscan Color - 120 Clocks - Sprites 7 and 8
;---------------------------------------------------------------
		; Left overscan color
;		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
;		mov W0, LATB					; (1)
		
PIXEL_LOOP:     
;-------------------------------------------------------------------------------------------------
;256 = 7 clocks a pixel (=1792 clocks)
;240 = 8 clocks a pixel
;192 = 10 clocks a pixel
;160 = 12 clocks a pixel
        mov #2, W7     ; (1) how many sprites to draw in a loop
        ; 2 sprites, 126 clocks
		END_SPRITE_7:
        ;draw sprite (60) --- Second sprite (63 since we are branching to draw two sprites)
        add #Sprite_Ptr_Size, W9        ;(1) advance sprite pointer to get info for this sprite
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c6 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c6  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite - 48 cycles
        ;pixel 0
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W2  ;(1) get the sprite color
        cpseq W2, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W2, [W3+15]  ;(1)
		END_SPRITE_6:
        sub W7, #1, W7;  (1) - subtract 1 from sprite count
		bra nz, END_SPRITE_7	; (1/2) check to see if we are drawing more sprites
		goto Setup_scanline ;(2)
;-- Time to kill if we aren't drawing the second or 5th sprite
No_sprite_1c6:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(60-5-5)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
        goto END_SPRITE_6 ;(2)
No_sprite_2c6:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(60-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_6 ;(2)
        
; W0 - color to write to screen
; W1 - pointer to TileSet
; W2 - counts how many pixels we have left to draw in the scanline
; W3 - counter for how many pixels we have left to draw for tile
; W4 - horizontal offset for tile
; W5 - address for current tile map
; W6 - address for current tile map * 64 (tile offset)
; W7 - byte for tile from tilemap
;5+18+126
;----- Set up scanline for tiles (18 cycles)
      Setup_scanline:
	    mov #240, W2; (1) pixel counter for number of pixels to draw
	    ;mov #192, W2; (1) pixel counter for number of pixels to draw
		mov _g_TileSet, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
        mov #0, W3; (1)
        mov _g_RasterLine, W4 ; (1) grab current scanline
        and #0x07, W4         ;(1) mask it off so we can use it for tiles
        sl W4, #3, W4            ;(1) multiply by 8 for offset
        add W1, W4, W1            ;(1) set new offset for tile
        mov _g_RasterLine, W6     ;(1) get current scanline to calc tile map address
		asr W6, #3, W6            ;(1) divide by 8 to get the tile row
		mov #30, W5                ;(1) Value we are multiplying by
		mul.us W6, W5, W6        ;(1) multiply by 30 (number of horizontal tiles)
		mov W6, W5               ;(1) move back to original register
        mov _g_TileMapBasePtr, W6 ;(1) for adding
		add W5, W6, W5        ;(1) get address for tile map
		mov #0, W7 ;(1) clear upper 8 bits of W6 register for tile byte (make sure it is only an 8 bit value)
        ;mov _g_ColorTable, W9; (1) get address to sprite scanline
        mov #ScanLine, W9 ;(1) get adddress to sprite scanline
        add #16, W9           ; (1) add 16 since that is the first visible pixel
		mov #0, W10          ;(1) for compare (if sprite pixel is zero then load graphics tile)
        ;finish out overscan ----
        ;Clock count: 2 for top of code, 2 for this loop,  60-2 is for sprite drawing,18 is setting up tiles
        ;repeat #(120-2-2-60-2-18)        ; (1) 
        ;Clock count: 2 for top of code, 2 for this loop, 126-2 is for sprite drawing,18 is setting up tiles
        ;repeat #(136-2-2-126-2-18)        ; (1) 
		;nop						; (1+n)
        nop  ;(1)
        nop  ;(1)
        nop  ;(1)
;1920 clocks from overscan-to-overscan
;---- Place rendering engine here (1920 free clocks)
Draw_Pixel:
	    ;8 clocks a pixel (240 pixels)
		; unroll loop for a full tile (8 pixels)
;pixel 0--------------------- (note: the sprite pixel line check throws off the pixel clock pushing it 2 clocks down)
		mov.b [W5], W7 ; (1) get tile we are rendering from tile map
        sl W7, #6, W6;   (1) multiply by 64 (each tile is 64 bytes)
        add W6, W1, W6  ;(1) combine tile graphics offset to tile image
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer--
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel--
        mov.b [W6],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        add #1, W5     ;(1) advance to next tile
		;nop ;(1)
        ;nop ;(1)
;pixel 1---------------------
        nop ;(1)
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 0
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+1],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 1
		nop ;(1)
        nop ;(1)
;pixel 2---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+2],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 2
		nop ;(1)
        nop ;(1)
;pixel 3---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
		cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 3
		nop ;(1)
        nop ;(1)
;pixel 4---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+4],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 4
		nop ;(1)
        nop ;(1)
;pixel 5---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+5],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 5
		nop ;(1)
        nop ;(1)
;pixel 6---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        nop ;(1)
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+6],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 6
		nop ;(1)
        nop ;(1)
;pixel 7---------------------
        mov.b [W9++], W0 ;(1) copy pixel from pixel buffer
        mov.b W10, [W9-1]; ;(1) erase scanline value for pixel 7
        cpsne W0, W10 ;(1/2) if pixel is not zero, then skip instruction to read tile pixel
        mov.b [W6+7],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
        sub W2, #8, W2;  (1) - subtract 8 from amount of pixels left to draw
		bra nz, Draw_Pixel	; (1/2) check to see if we draw more pixels
;---------------------------
; Draw righthanded overscan
;---------------------------
		; Now set PORTB to overscan
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		nop				; (1)
		nop				; (1)
		mov W0, LATB			; (1)

;-------------------------------------------------------------------------------------------------
; Draw 1st sprite on scanline
;-------------------------------------------------------------------------------------------------
      ;set up scanline for all sprites (done once per scanline) - 7 cycles
        mov _g_SpriteList, W9; (1) pointer for sprite data        
        mov _g_RasterLine, W8; (1) find out which scanline we are using
        add W8, #1, W8       ; (1) add one since we are drawing this for the next scanline
        add W8, #16, W8 ; (1) add this so we can move sprite offscreen on the Y axis
        mov #0, W4     ; (1) make sure that upper 8 bits are clear for sprite Y value
        mov #0, W5     ; (1) make sure that upper 8 bits are clear for sprite X value
        mov #0, W10    ; (1) used for compare for sprite
      ;draw sprite (29) ---- First sprite (27 and 2 for goto)
        mov.b [W9+4], W4 ;(1) get Y value for sprite
        sub W8, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		BRA n, No_sprite_1c0 ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
        BRA GT, No_sprite_2c0  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W6  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov #ScanLine, W3 ;(1) get adddress to sprite scanline
        mov.b [W9+2], W5 ;(1) get x value for sprite
	    mov [W9+6], W1      ;(1) sprite graphic address
        add W1, W6, W1     ;(1) add scanline offset to get correct sprite line
        add W3, W5, W3    ;(1) add x value to sprite scanline 
        ;unrolled loop for drawing sprite -  16 cycles
;;      This will place sprite without regards to transparency
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1++], [W3++]   ;(1)
        mov.b [W1], [W3]   ;(1)

		END_SPRITE_0:
        goto Finish_Scanline
;-- Time to kill if we aren't drawing the first sprite
No_sprite_1c0:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(27-5-5)    ;(1)
        nop               ;(1+n)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
;        goto Setup_scanline ;(2)
        goto END_SPRITE_0 ;(2)
No_sprite_2c0:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(27-7-4)    ;(1)
        nop               ;(1+n)
;        goto Setup_scanline ;(2)
        goto END_SPRITE_0 ;(2)
        Finish_Scanline:
;-------------------------------------------------------------------------------------------------
        inc _g_RasterLine                       ; (1) Increment our raster line count
		; Done with this completely
FINISHED_NTSC:

        bclr IFS0, #T1IF           		; Clear the Timer1 Interrupt flag Status
                                   		; bit.

		pop RCOUNT			; (1) Make sure we are pushing Rcount since we are using repeat
		pop W5
		pop W4                   	; Retrieve context POP-ping from Stack
		pop SR
		pop.s				; (1) Push status registers

		disi #0x0			; Re-enable interrupts

        RETFIE                     		;Return from Interrupt Service routine




;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
        